package Control;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.*;
import java.util.StringTokenizer;
import java.lang.Math;

import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import Model.Application;
import Model.Direction;
import Model.Type;
import Model.Direction.InitType;
import View.View;

// TODO: Auto-generated Javadoc
/* Continuity Project v0.0.1.0
 *  2012 MzliTeam
 *  Berki Ramna
 *  Gerlei Balzs
 *  Halsz Antal Istvn
 *  Kiss Beta
 *  Szab Csaba
 * 
 * Konzulens: Dr. Lszl Zoltn
 */

/**
 * Controller osztly.
 * 
 * Ez jn ltre elszr a program indtsakor. Felels az idkezelsrt s a
 * csempe vagy bbu mozgatsrt.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Controller extends JFrame implements KeyListener, ActionListener {

	/** A futtatand program. */
	private static Application app;

	/** A program fo szala. */
	private static MainThread mThread;

	/** A program mellek (idle) szala. */
	private static IdleThread iThread;

	/** The move tiles. */
	private static boolean moveTiles;

	public static boolean[] keyFlags;
	// keyFlags knnyebb indexelshez
	public static final int RIGHT = 0;
	public static final int UP = 1;
	public static final int LEFT = 2;
	public static final int DOWN = 3;
	public static final int CTRL = 4;
	public static final int D = 5;
	public static final int W = 6;
	public static final int A = 7;
	public static final int S = 8;
	public static final int SPACE = 9;

	/** Menu elemeit tartalmazo kontener. */
	private static JPanel panel;
	
	/** Sugo elemeit tartalmazo kontener. */
	private static JPanel helpPanel;
		
	/** Jatekok megjelenito view */
	private static View view;

	/** Hanyszor inditottuk el a jatekot */
	private static int gameStarted;
		
	/**
	 * Controller konstruktora. Ltrehoz egy j Controllert.
	 * 
	 * @param name
	 *            the name
	 */
	public Controller(String name) {
		super(name);
		app = new Application();
		panel = new JPanel();
		helpPanel = new JPanel();
		iThread = new IdleThread(app);
		moveTiles = true;
		gameStarted = 0;
		keyFlags = new boolean[10];
		for (boolean b : keyFlags) {
			b = false;
		}
	}

	
	
	/**
	 * A program main fggvnye.
	 * 
	 * @param args
	 *            parancssori argumentumok
	 */
	public static void main(String[] args) {
		mThread = new MainThread();
		mThread.setState(0);
		// csak a fo szalat inditom el, ami a menut megjeleniti eloszor
		mThread.start();
	}

	/**
	 * A menu felepitese.
	 */
	void setupMenuView() {
		JLabel label = new JLabel();
		label.setFont(new java.awt.Font("Segoe UI", 1, 100));
		label.setText("Continuity");
		
		panel.setBackground(new Color(153, 255, 0));
		Color col = new Color(255, 255, 0);

		JButton menuButton1 = new JButton();
		menuButton1.setFont(new java.awt.Font("Segoe UI", 0, 36));
		menuButton1.setText("\u00daj j\u00e1t\u00e9k");
		menuButton1.addActionListener(this);
		menuButton1.setBackground(col);
		
		JButton menuButton2 = new JButton();
		menuButton2.setFont(new java.awt.Font("Segoe UI", 0, 36));
		menuButton2.setText("J\u00e1t\u00e9k folytat\u00e1sa");
		menuButton2.addActionListener(this);
		menuButton2.setBackground(col);
		
		JButton menuButton3 = new JButton();
		menuButton3.setFont(new java.awt.Font("Segoe UI", 0, 36));
		menuButton3.setText("S\u00fag\u00f3");
		menuButton3.addActionListener(this);
		menuButton3.setBackground(col);
		
		JButton menuButton4 = new JButton();
		menuButton4.setFont(new java.awt.Font("Segoe UI", 0, 36));
		menuButton4.setText("Kil\u00e9p\u00e9s");
		menuButton4.addActionListener(this);
		menuButton4.setBackground(col);
		
		GroupLayout layout = new javax.swing.GroupLayout(panel);
		panel.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addContainerGap(170, Short.MAX_VALUE)
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
										.addComponent(label)
										.addGap(136,136,136))
										.addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
												.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING,false)
														.addComponent(menuButton1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
														.addComponent(menuButton2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
														.addComponent(menuButton3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
														.addComponent(menuButton4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
														.addGap(238, 238, 238)))));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout
						.createSequentialGroup()
						.addGap(36, 36, 36)
						.addComponent(label)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(menuButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
						.addComponent(menuButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 92, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
						.addComponent(menuButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 88, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(menuButton4, javax.swing.GroupLayout.DEFAULT_SIZE, 84, Short.MAX_VALUE)
						.addGap(29, 29, 29)));
		
		panel.setFocusable(true);

		getContentPane().add(panel);
	}

	/**
	 * Disable menu view.
	 */
	void disableMenuView() {
		panel.setVisible(false);
	}
	
	public void enableMenuView() {
		panel.setVisible(true);
	}
	
	public void disableGameView() {
		view.setVisible(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent event) {

		// FigureA
		// ha bal nyilat nyomott le
		if (event.getKeyCode() == 37) {
			//("BAL");
			if (moveTiles == false) {
				keyFlags[LEFT] = true;
			} else {
				app.move(Type.Tile,
						new Direction(-10, 0, InitType.Coordinates), null);
			}
		}
		// ha jobb nyilat nyomott le
		else if (event.getKeyCode() == 39) {
			//("JOBB");
			if (moveTiles == false) {
				keyFlags[RIGHT] = true;
			} else {
				app.move(Type.Tile, new Direction(10, 0, InitType.Coordinates),
						null);
			}
		}
		// ha felfele nyilat nyomott le
		else if (event.getKeyCode() == 38) {
			//("FEL");
			if (moveTiles == false) {
				keyFlags[UP] = true;
			} else {
				app.move(Type.Tile, new Direction(0, 10, InitType.Coordinates),
						null);
			}
		}
		// ha lefele nyilat nyomott le
		else if (event.getKeyCode() == 40) {
			//("LE");
			if (moveTiles == true) {
				keyFlags[DOWN] = true;
				app.move(Type.Tile,
						new Direction(0, -10, InitType.Coordinates), null);
			}
		}
		// ha controlt nyomott le (csempemozgat mdba vlts)
		else if (event.getKeyCode() == 17) {
			//("CTRL");
			keyFlags[CTRL] = true;
			if (moveTiles == true)
				moveTiles = false;
			else
				moveTiles = true;
		}

		// FigureB
		// ha A-t nyomott le
		if (event.getKeyCode() == 65) {
			//("A");
			if (moveTiles == false) {
				keyFlags[A] = true;
			} else {
				app.move(Type.Tile,
						new Direction(-10, 0, InitType.Coordinates), null);
			}
		}
		// ha D-t nyomott le
		else if (event.getKeyCode() == 68) {
			//("D");
			if (moveTiles == false) {
				keyFlags[D] = true;
			} else {
				app.move(Type.Tile, new Direction(10, 0, InitType.Coordinates),
						null);
			}
		}
		// ha W-t nyomott le
		else if (event.getKeyCode() == 87) {
			//("W");
			if (moveTiles == false) {
				keyFlags[W] = true;
			} else {
				app.move(Type.Tile, new Direction(0, 10, InitType.Coordinates),
						null);
			}
		}
		// ha S-t nyomott le
		else if (event.getKeyCode() == 83) {
			//("S");
			if (moveTiles == true) {
				keyFlags[S] = true;
				app.move(Type.Tile,
						new Direction(0, -10, InitType.Coordinates), null);
			}
		}
		// ha space-t nyomott le (csempemozgat mdba vlts)
		else if (event.getKeyCode() == 32) {
			//("SPACE");
			keyFlags[SPACE] = true;
			if (moveTiles == true)
				moveTiles = false;
			else
				moveTiles = true;
		}

		//TODO ha Esc gombot nyomott le (mentes)
		else if (event.getKeyCode() == 27) {
			//("MENTS S KILPS");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent event) {
			// FigureA
			// ha bal nyilat engedett fel
			if (event.getKeyCode() == 37) {
				//("BAL release");
					keyFlags[LEFT] = false;
			}
			// ha jobb nyilat engedett fel
			else if (event.getKeyCode() == 39) {
				//("JOBB release");
					keyFlags[RIGHT] = false;
			}
			// ha felfele nyilat engedett fel
			else if (event.getKeyCode() == 38) {
					//("FEL release");
					keyFlags[UP] = false;
			}
			// ha lefele nyilat engedett fel
			else if (event.getKeyCode() == 40) {
				//("LE release");
					keyFlags[DOWN] = false;
			}
			// ha controlt engedett fel
			else if (event.getKeyCode() == 17) {
				//("CTRL release");
				keyFlags[CTRL] = false;
			}

			// FigureB
			// ha A-t engedett fel
			if (event.getKeyCode() == 65) {
				//("A release");
					keyFlags[A] = false;
			}
			// ha D-t engedett fel
			else if (event.getKeyCode() == 68) {
				//("D release");
					keyFlags[D] = false;
			}
			// ha W-t engedett fel
			else if (event.getKeyCode() == 87) {
				//("W release");
					keyFlags[W] = false;
			}
			// ha S-t engedett fel
			else if (event.getKeyCode() == 83) {
				//("S release");
					keyFlags[S] = false;
			}
			// ha space-t engedett fel (csempemozgat mdba vlts)
			else if (event.getKeyCode() == 32) {
				//("SPACE release");
				keyFlags[SPACE] = false;
			}

			//TODO ha Esc gombot engedett fel (mentes)
			else if (event.getKeyCode() == 27) {
				//("MENTS S KILPS");
				//disableGameView();
				enableMenuView();
				iThread.setCanRun(false);
				
				try {
					FileOutputStream f = new FileOutputStream("gameState");
					ObjectOutputStream out = new ObjectOutputStream(f);
					out.writeObject(app.getActualLevelPath());
					out.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent arg0) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().matches("\u00daj j\u00e1t\u00e9k")) {
			// Uj jatek inditasa
			
			disableMenuView();
			// atattiljuk az mThread allapotat jatekra
			mThread.setState(1);

			// betoltjuk az elso palyat
			try {
				PrintWriter pw = new PrintWriter(new FileWriter(
						"szint1_output.txt"));
				app.loadLevel(-1, "szint1.txt", pw);
				pw.close();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			
			app.setupView(view, this, iThread);

			// elinditjuk az idle Threadet ami az idot szamolja
			if(gameStarted == 0) {
				iThread.start();
				iThread.setCanRun(true);
				gameStarted++;
			} else {
				iThread.setCanRun(true);
				gameStarted++;
			}

		} else if (e.getActionCommand().matches("J\u00e1t\u00e9k folytat\u00e1sa")) {
			// Uj jatek inditasa
			
						disableMenuView();
						// atattiljuk az mThread allapotat jatekra
						mThread.setState(1);

						String savedLevelPath = null;
						
						try {
							FileInputStream f = new FileInputStream("gameState");
							ObjectInputStream in;
							in = new ObjectInputStream(f);
							savedLevelPath = (String)in.readObject();
							in.close();
						} catch (IOException e2) {
							e2.printStackTrace();
						} catch (ClassNotFoundException e3) {
							e3.printStackTrace();
						}
						
						
						// betoltjuk az elso palyat
						try {
							PrintWriter pw = new PrintWriter(new FileWriter(
									"szint1_output.txt"));
							app.loadLevel(-1, savedLevelPath, pw);
							pw.close();
						} catch (IOException e1) {
							e1.printStackTrace();
						}
						
						app.setupView(view, this, iThread);

						// elinditjuk az idle Threadet ami az idot szamolja
						if(gameStarted == 0) {
							iThread.start();
							iThread.setCanRun(true);
							gameStarted++;
						} else {
							iThread.setCanRun(true);
							gameStarted++;
						}
		} else if (e.getActionCommand().matches("S\u00fag\u00f3")) {
			disableMenuView();
			helpPanel.setLayout(new BoxLayout(helpPanel, BoxLayout.PAGE_AXIS));
			JTextArea ta = new JTextArea(new String("A jtk clja:\n\tA sajt bbunkkal szedjk fel a plyn lv sszes kulcsot, s rjk el a kijratot, ezzel teljestjk a szintet.\n\nA jtk menete:\n\tKt jtkos jtszik egyms ellen a piros s a kk bbuk setsgvel. Tudjk a bbuikat s a csempket is mozgatni.\n\tHa az egyik jtkos bbumozgat mdban van, akkor a msik jtkos is csak bbut mozgathat, amg mdot nem vlt s fordtva.\n\tA csempket gy prbljuk elhelyezni a plyn, hogy minl egyszerbben tudjunk a kulcshoz majd a kijrathoz eljutni.\n\tKzben a msik jtkos mozgst prbljuk akadlyozni.\n\nA parancsok:\n\tSpace s Control: Mdvlts.\n\tFel : A kk bbu felfel mozagtsa.\n\tLe : A kk bbu lefel mozagtsa.\n\tJobb : A kk bbu jobbra mozagtsa.\n\tBal : A kk bbu balra mozagtsa.\n\tW : A piros bbu felfel mozagtsa.\n\tS : A piros bbu lefel mozagtsa.\n\tD : A piros bbu jobbra mozagtsa.\n\tA : A piros bbu balra mozagtsa.\n\tUgorni a fel s a jobb vagy bal irny gombokkal lehet.\n"));
			ta.setEditable(false);
			ta.setBackground(new Color(204, 255,255));
			helpPanel.add(ta);
			JButton vissza = new JButton("Vissza");
			helpPanel.add(vissza);
			vissza.addActionListener(this);
			
			getContentPane().add(helpPanel);
			
		} else if (e.getActionCommand().matches("Kil\u00e9p\u00e9s")) {
			System.exit(1);
		}else if (e.getActionCommand().matches("Vissza")) {
			getContentPane().remove(helpPanel);
			 helpPanel.removeAll();
			enableMenuView();
		}
		
	}

	static public Direction calculateDirection(Type t) {
		//Figure A
		if (t == Type.FigureA) {
			// RIGHT
			if (keyFlags[RIGHT] == true && keyFlags[LEFT] == false
					&& keyFlags[UP] == false) {
				return new Direction(5, 0, InitType.Coordinates);
			}
			// UP
			else if (keyFlags[RIGHT] == false && keyFlags[LEFT] == false
					&& keyFlags[UP] == true) {
				return new Direction(0, 24, InitType.Coordinates);
			}
			// LEFT
			else if (keyFlags[RIGHT] == false && keyFlags[LEFT] == true
					&& keyFlags[UP] == false) {
				return new Direction(-5, 0, InitType.Coordinates);
			}
			// RIGHT & UP
			else if (keyFlags[RIGHT] == true && keyFlags[LEFT] == false
					&& keyFlags[UP] == true) {
				return new Direction(7, 24, InitType.Coordinates);
			}
			// LEFT & UP
			else if (keyFlags[RIGHT] == false && keyFlags[LEFT] == true
					&& keyFlags[UP] == true) {
				return new Direction(-7, 24, InitType.Coordinates);
			} else {
				return new Direction();
			}
		} 
		//Figure B
		else if (t == Type.FigureB) {
			// D
			if (keyFlags[D] == true && keyFlags[A] == false
					&& keyFlags[W] == false) {
				return new Direction(5, 0, InitType.Coordinates);
			}
			// W
			else if (keyFlags[D] == false && keyFlags[A] == false
					&& keyFlags[W] == true) {
				return new Direction(0, 24, InitType.Coordinates);
			}
			// A
			else if (keyFlags[D] == false && keyFlags[A] == true
					&& keyFlags[W] == false) {
				return new Direction(-5, 0, InitType.Coordinates);
			}
			// D & W
			else if (keyFlags[D] == true && keyFlags[A] == false
					&& keyFlags[W] == true) {
				return new Direction(7, 24, InitType.Coordinates);
			}
			// A & W
			else if (keyFlags[D] == false && keyFlags[A] == true
					&& keyFlags[W] == true) {
				return new Direction(-7, 24, InitType.Coordinates);
			} else {
				return new Direction();
			}
		}
		
		return new Direction();
	}
}