package Model;

import java.awt.Point;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;

/**
 * Exit osztly.
 * 
 * Ezt kell elrni a megfelel kulcsokkal a birtokunkban a szint teljestshez.
 * Tudja, hogy hny kulcsra van szksge. Ha nincs elg kulcs a bbunl, akkor
 * megy tovbb a bbu.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Exit extends GameObject {
	/** A szint teljestshez szksges kulcsok szma. */
	private int numberOfKeys;

	/**
	 * Exit default konstruktora. Ltrehoz egy j kijratot.
	 */
	public Exit(Point point, FullTile tile) {
		super(point, tile);
		numberOfKeys = 1;
	}

	/**
	 * Exit paramteres konstruktora. Lterhoz egy j kijratot.
	 * 
	 * @param numOfKeys
	 *            a szksges kulcsok szma.
	 */
	public Exit(Point point, int numOfKeys,  FullTile tile) {
		super(point, tile);
		numberOfKeys = numOfKeys;
	}

	/**
	 * Megvizsglja a kapott bbu zsebben lv kulcsokat. Ha megfelel szm
	 * van, akkor true-val tr vissza, ha nem akkor false-szal.
	 * 
	 * @param figure
	 *            the bbu
	 * @return Igaz, ha kijratot tallt, s vge a szintnek.
	 */
	public boolean interact(Figure f,String name, PrintWriter pw) {
		if(f.getPocket().size() == numberOfKeys){
			if(pw!=null)
			pw.println(name +" kijarathoz ert, es van eleg kulcsa. A szint teljesitve!");
			return true;
		}
		if(pw!=null)
		pw.println(name +" kijarathoz ert, de nincs eleg kulcsa.");
		return false;
	}

	/**
	 * Visszaadja a szint teljestshez szksges kulcsok szmt.
	 * 
	 * @return a szksges kulcsok szma
	 */
	public int getNumberOfKeys() {
		return numberOfKeys;
	}

	@Override
	public Type whatIsIt() {
		return Type.Exit;
	}
}
