package Model;

import java.awt.Point;
import java.io.PrintWriter;
import java.util.*;

/**
 * Figure osztly.
 * 
 * A felhasznl ltal irnytott bbut valstja meg. Tud mozogni. Kpes
 * megadni a szksges adatokat a pontos elmozduls-szmtshoz, valamint kpes
 * visszaadni a zsebt.
 * 
 * @author Szab Csaba
 * @author Gerlei Balzs
 * @author Halsz Antal Istvn
 * @author Berki Ramna
 * @version 1.0
 */
public class Figure {
	/** A bbu koordintja. */
	private Point coord;

	/**
	 * a felhasznli input ltal belltott irnyvektor, ebbe az irnyba akar
	 * elmozdulni a bbu.
	 */
	private Direction dir;

	/**
	 * Referencia arra a csempre, amiben ppen van a bbu.
	 */
	private FullTile myTile;

	/**
	 * Az elz lpsben kiszmtott elmozduls vektor.
	 */
	private Direction oldDirection;

	/** Az elz simulate futsakor belltott felhasznli irnyvektor. */
	private Direction oldInputDir;

	/** Trolja, hogy az elz simulate-kor kapott-e irnyvektort. */
	private boolean lastTimeInput;

	/**
	 * A bbu zsebe, melyben kulcsokat trolunk.
	 */
	private ArrayList<Key> pocket;

	/**
	 * Figure konstruktora. Ltrehoz egy j bbut.
	 * 
	 * @param tile
	 *            a szl csempe
	 */
	public Figure(FullTile tile, Point startCoord) {
		coord = startCoord;
		dir = new Direction();
		myTile = tile;
		oldDirection = new Direction();
		oldInputDir = new Direction();
		lastTimeInput = false;
		pocket = new ArrayList<Key>();
	}

	/**
	 * Visszaadja a bbu koordintjt.
	 * 
	 * @return a bbu koordintja
	 */
	public Point getCoord() {
		return coord;
	}

	/**
	 * Visszaadja az input alapjn belltott irnyvektort.
	 * 
	 * @return a bbu irnya
	 */
	public Direction getDirection() {
		return dir;
	}

	/**
	 * Visszaadja, hogy elz simulate-kor kapott-e a bbu irnyvektort.
	 * 
	 * @return igaz, ha elz simulate-kor a bbu kapott irnyvektort
	 */
	public boolean getLastTimeInput() {
		return lastTimeInput;
	}

	/**
	 * Visszaadja a csempt, amiben a bbu van.
	 * 
	 * @return a bbut tartalmaz csempe
	 */
	public FullTile getMyTile() {
		return myTile;
	}

	/**
	 * Visszaadja az elz lpsben szmtott elmozdulsvektort.
	 * 
	 * @return A bbu korbbi irnya.
	 */
	public Direction getOldDirection() {
		return oldDirection;
	}

	/**
	 * Visszaadja az elz lps inputirnyt.
	 * 
	 * @return az elz lps inputirnya
	 */
	public Direction getOldInputDir() {
		return oldInputDir;
	}

	/**
	 * Visszaadja a bbu zsebt.
	 * 
	 * @return A bbu zsebe.
	 */
	public ArrayList<Key> getPocket() {
		return pocket;
	}

	/**
	 * Mozgatja a bbut.
	 * 
	 * @param dt
	 *            Utols mozgats ta eltelt id.
	 */
	public void move(int dt, String name, PrintWriter pw) {
		Point oldCoord = new Point(coord);
		//elmozdul a bbu
		
		for (int i = 1; i <= dt; ++i) {
			coord = myTile.moveFigure(this, name, pw);
		}
		//miutn elmozdult, kinullzza az inputot
		this.setDirection(new Direction());
		if (oldCoord.x != coord.x || oldCoord.y != coord.y) {
			if (pw != null)
				pw.println(name + " a(z) " + myTile.getTileIndex().x + " "
						+ myTile.getTileIndex().y + " csempeben a(z) "
						+ coord.x + " " + coord.y + " koordinatara kerult.");
		}else{
			if (pw != null)
				pw.println(name + " maradt a(z) " + myTile.getTileIndex().x + " "
						+ myTile.getTileIndex().y + " csempeben a(z) "
						+ coord.x + " " + coord.y + " koordinatan.");
		}
	}

	/**
	 * Belltja a bbu koordintjt.
	 * 
	 * @param point
	 *            az j koordinta, amire be akarjuk lltani a figurt
	 */
	public void setCoord(Point point) {
		// Belltjuk az j koordintt
		coord = point;
	}

	/**
	 * Belltja az input alapjn a kvnt elmozduls irnyvektort.
	 * 
	 * @param dir
	 *            a belltand j irny
	 */
	public void setDirection(Direction newDir) {
		// j irny belltsa
		dir = new Direction(newDir);
	}

	/**
	 * Belltja, hogy elz simulate-kor volt-e inputirnya a bbunak.
	 * 
	 * @param az
	 *            j llapot
	 */
	public void setLastTimeInput(boolean statement) {
		lastTimeInput = statement;
	}

	/**
	 * Belltja a csempre mutat referencit.
	 * 
	 * @param tile
	 *            az j sajtcsempe
	 */
	public void setMyTile(FullTile tile) {
		// j sajt csempe belltsa
		myTile = tile;
	}

	/**
	 * Belltja a kiszmtott elmozduls vektort.
	 * 
	 * @param dir
	 *            A bbu rgi irnya.
	 */
	public void setOldDirection(Direction oldDir) {
		// Bbu rgi irnynak belltsa
		oldDirection = oldDir;
	}

	public void setOldInputDir(Direction oldInput) {
		oldInputDir = oldInput;
	}
	
	/**
	 * A modell beli koordintt a kirajzolsnak megfelel
	 * koordinta-rendszerbe vltja t.
	 * 
	 * @return kirajzolshoz hasznlhat koordinta
	 */
	public Point transformToMatrixCoord() {
		return new Point(coord.x, 400 - coord.y);
	}
}
